﻿using System;
using System.Collections.Generic;
using System.IO;
using System.Linq;

namespace Hangman
{
    class Program
    {
        static void Main(string[] args)
        {
            int allowedIncorrectCharacters = 8;
            string[] words = File.ReadAllLines("Words.txt");
            Random random = new Random();

            string word = words[random.Next(words.Length)].ToUpper(); // Always deal with only uppercase characters.

            List<char> guessedCharacters = new List<char>();
            List<char> incorrectCharacters = new List<char>();

            Console.WriteLine("Welcome to the Pokemon Hangman Game!");
            Console.WriteLine("I'm thinking of a Pokemon. Guess letters to fill in the name. 8 incorrect guesses and you lose!");

            while(HasUnguessedCharacter(word, guessedCharacters) && incorrectCharacters.Count < allowedIncorrectCharacters)
            {
                DisplayStatus(word, guessedCharacters, incorrectCharacters, allowedIncorrectCharacters);
                char newCharacter = GetNewCharacterFromUser(guessedCharacters);
                guessedCharacters.Add(newCharacter);
                if (!word.Contains(newCharacter))
                    incorrectCharacters.Add(newCharacter);
            }


            DisplayStatus(word, guessedCharacters, incorrectCharacters, allowedIncorrectCharacters);

            if (incorrectCharacters.Count >= allowedIncorrectCharacters)
                Console.WriteLine($"You lose. My word was '{word}'.");
            else
                Console.WriteLine($"You win! You correctly guessed my word: {word}.");

            Console.ReadKey();
        }

        private static bool HasUnguessedCharacter(string word, List<char> guessedCharacters)
        {
            for (int index = 0; index < word.Length; index++)
                if (!guessedCharacters.Contains(word[index]))
                    return true;
            return false;
        }

        private static void DisplayStatus(string word, List<char> guessedCharacters, List<char> incorrectCharacters, int allowedIncorrectCharacters)
        {
            Console.WriteLine("\n");
            for(int index = 0; index < word.Length; index++)
            {
                if (guessedCharacters.Contains(word[index])) Console.Write(word[index]);
                else Console.Write("_");
                Console.Write(" ");
            }
            Console.WriteLine();

            Console.Write($"Incorrect Guesses ({incorrectCharacters.Count}/{allowedIncorrectCharacters}): ");
            foreach (char incorrect in incorrectCharacters)
                Console.Write(incorrect + " ");
            Console.WriteLine("\n");
        }

        private static char GetNewCharacterFromUser(List<char> guessedCharacters)
        {
            while(true)
            {
                Console.Write("Enter a character:  ");
                string input = Console.ReadLine().ToUpper(); // always deal with only uppercase characters.
                if (input.Length == 1 && !guessedCharacters.Contains(input[0]))
                    return input[0];

                // Failure cases.
                if (input.Length != 1) Console.WriteLine("You must enter a single character.");
                else if (guessedCharacters.Contains(input[0])) Console.WriteLine("You have already entered that character.");
            }
        }
    }
}
